import java.util.Random;
import java.awt.Color;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * Ein einfacher Game of Life-Simulator.
 * 
 * @author Ronald Ortner
 * @version 2015.01.16
 */
public class Simulator implements ActionListener, FocusListener
{
    // Konstanten fuer Konfiguration der Simulation.
    // Die Standardbreite fuer ein Feld.
    private static final int STANDARD_BREITE = 100;
    // Die Standardlaenge fuer ein Feld.
    private static final int STANDARD_LAENGE = 100;
    // Die Standardwahrscheinlichkeit fuer eine lebende Zelle.
    private static final double STANDARD_PROB = 0.1;


    // Der aktuelle Zustand der Welt
    private Welt welt;
    // Zweites Feld, mit dem der naechste Schritt erzeugt wird
    private Welt zwelt;
    // Der aktuelle Schritt der Simulation
    private int schritt;
    // Eine grafische Ansicht der Simulation
    private Simulationsansicht ansicht;
    
    // graphische Elemente im Fenster
    private JButton startknopf;
    private JTextField eingabefeld;
    
    /**
     * Erzeuge ein Simulationsfeld mit einer Standardgr��e.
     */
    public Simulator()
    {
        this(STANDARD_BREITE,STANDARD_LAENGE,STANDARD_PROB);
    }
    
    /**
     * Erzeuge ein Simulationsfeld mit der gegebenen Groesse.
     * @param tiefe die Laenge des Feldes (muss groesser als 0 sein).
     * @param breite die Breite des Feldes (muss groesser als 0 sein).
     */
    public Simulator(int laenge, int breite, double prob)
    {
        if(breite <= 0 || laenge <= 0) {
            System.out.println("Abmessungen muessen groesser als 0 sein.");
            System.out.println("Benutze Standardwerte.");
            laenge = STANDARD_LAENGE;
            breite = STANDARD_BREITE;
        }
        
        welt = new Welt(laenge, breite);
        zwelt = new Welt(laenge, breite);

        // Eine Ansicht der Zustaende aller Positionen im Feld erzeugen.
        ansicht = new Simulationsansicht(laenge, breite);
        zeigeGraphik();
        
        schritt = 0;
        welt.leere();
        zwelt.leere();
        bevoelkere(welt,prob);
        
        // Zeige den Startzustand in der Ansicht.
        ansicht.zeigeStatus(schritt, welt);
    }
    
    /**
     * Starte die Simulation vom aktuellen Zustand aus fuer einen laengeren
     * Zeitraum, etwa 500 Schritte.
     */
    public void starteLangeSimulation()
    {
        simuliere(5000);
    }
    
    /**
     * Fuehre vom aktuellen Zustand aus die angegebene Anzahl an
     * Simulationsschritten durch.
     * Brich vorzeitig ab, wenn die Simulation nicht mehr aktiv ist.
     * @param schritte Anzahl der zu simulierenden Schritte.
     */
    public void simuliere(int schritte)
    {
        for(int schritt = 1; schritt <= schritte && ansicht.istAktiv(welt); schritt++) {
            simuliereEinenSchritt();
        }
    }
    
    /**
     * Fuehre einen einzelnen Simulationsschritt aus:
     * Durchlaufe alle Feldpositionen und aktualisiere den 
     * Zustand jeder Zelle.
     */
    public void simuliereEinenSchritt()
    {
        schritt++;
        
        // Regeln des Game of Life
        // Any live cell with fewer than two live neighbours dies, as if caused by under-population.
        // Any live cell with two or three live neighbours lives on to the next generation.
        // Any live cell with more than three live neighbours dies, as if by overcrowding.
        // Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.
        
        for(int zeile=0; zeile<welt.gibLaenge(); zeile++){
            for(int spalte=0; spalte<welt.gibBreite(); spalte++){
                
                int anzN = welt.anzLebendeNachbarn(zeile,spalte);
                
                if(welt.gibWelt(zeile,spalte)==1)
                {
                    if(anzN>=2 && anzN<=3)
                        zwelt.set(zeile,spalte,1);
                    else
                        zwelt.set(zeile,spalte,0);
                }else 
                if(anzN==3)
                    zwelt.set(zeile,spalte,1);
             }
         }
        // welt und zwelt am Ende des Schritts austauschen.
        Welt temp = welt;
        welt = zwelt;
        zwelt = temp;
        zwelt.leere();

        // Das neue Feld in der Ansicht anzeigen.
        ansicht.zeigeStatus(schritt, welt);
    }
          
 
    
    /**
     * Bevoelkere die Welt mit Lebewesen.
     * @param welt: die zu bevoelkernde Welt; prob: Wahrscheinlichkeit fuer lebende Zelle
     */
    private void bevoelkere(Welt welt, double prob)
    {
        Random rand = new Random();
        welt.leere();
        for(int zeile = 0; zeile < welt.gibLaenge(); zeile++) {
            for(int spalte = 0; spalte < welt.gibBreite(); spalte++) {
                if(rand.nextDouble() <= prob)
                    welt.set(zeile,spalte,1);
                else 
                    welt.set(zeile,spalte,0);
            }
        }
    }
    
    
    /**
     * Graphik anzeigen
     */
    private void zeigeGraphik()
    {
       
        // definiert Button und Textfeld
        startknopf = new JButton("Start");
        eingabefeld = new JTextField("Anzahl Simulationsschritte");
        
        // damit Button und Textfeld auf Druecken reagieren
        startknopf.addActionListener(this);
        eingabefeld.addActionListener(this);
        eingabefeld.addFocusListener(this);
        
        // Button und Textfeld ins Fenster geben
        Container inhalt = ansicht.getContentPane();
        inhalt.add(startknopf,BorderLayout.EAST);
        inhalt.add(eingabefeld,BorderLayout.SOUTH);
        
        // Fenster sichtbar machen
        ansicht.pack();
        ansicht.setVisible(true);
    }
    
       
    /**
     * Diese Methode aus dem Interface ActionListener wird ausgefuehrt, 
     * wenn Button gedrueckt oder was im Textfeld Enter gedrueckt wird. 
     */
    public void actionPerformed(ActionEvent event)
    {
            
        int n = 1000;
        String eingabe = eingabefeld.getText();
        
        if(istZahl(eingabe))
            n = Integer.parseInt(eingabe); 
        else
            eingabefeld.setText("1000");
          
        final int fn = n;    
            
        // eine anonyme innere Klasse, die von Runnable erbt.
        // Diese Klasse bekommt keinen eigenen Namen.
        // Von ihr wird nur ein Objekt (mit new) erzeugt.
        // Dieses wird sofort an den Konstruktor von Thread weitergegeben.
        Thread t = new Thread(new Runnable() {
            public void run() {
                simuliere(fn);
            }
        });
        
        t.start();
            
    }
    
    /**
     * Diese Methode aus dem Interface FocusListener wird ausgefuehrt, 
     * wenn ins Textfeld geklickt wird. 
     */
    public void focusGained(FocusEvent e)
    {
        eingabefeld.setText("");
    }
    
    /**
     * Diese Methode aus dem Interface FocusListener wird ausgefuehrt, 
     * wenn aus dem Textfeld geklickt wird. 
     */
    public void focusLost(FocusEvent e)
    {
        String eingabe = eingabefeld.getText();
        if(!istZahl(eingabe))
            eingabefeld.setText("100");
    }
    
    
    /**
     * ueberprueft ob String ein Integer ist
     */
    static boolean istZahl(String s)
    {
        try
        { 
            int i = Integer.parseInt(s); 
            return true; 
        }
        catch(NumberFormatException e)
        { return false; }
    }
    
    
    
    
}
